uab.classes.ClickActivator = uab.mixins.aceOO.Class()

function uab.classes.ClickActivator.prototype:init(statemultiplier, name, data)
	uab.classes.ClickActivator.super.prototype.init(self)

    self.statemultiplier = statemultiplier
	self.name = name
	self:SetActivationData(data)
	self.groups = {}

    if not data.groups.hostile then
        data.groups.hostile = {}
    end

    self.groups.hostile = uab.classes.HostileActions:new(self, data.groups.hostile)
    
    uab.GroupIterate(
        function(self, context, name, info)
            local grpname = info.name
            
            if not context.groups[grpname] then
                context.groups[grpname] = {}
            end
            self.groups[grpname] = uab.classes.UnitGroup:new(self, grpname, info.unitcount, context.groups[grpname])
        end,
        self,
        data)	

	if data.enabled then
	    self:Enable()
	else
	    self:Disable()
	end

    self:SecureHeaderChanged()
end

function uab.classes.ClickActivator.prototype:Dump(dumper)
    dumper:Dump("ClickActivator: "..self.name)
    dumper:IncIndent()
    dumper:Dump("Prefix: '"..self.prefix.."', Buttoncode: '"..self.buttoncode.."'")
    self.groups.hostile:Dump(dumper)
    uab.GroupIterate(
        function(self, context, name, info)
            self.groups[info.name]:Dump(context)
        end,
        self,
        dumper)
    dumper:DecIndent()
end

function uab.classes.ClickActivator.prototype:NewUnitGroupControl(unitgroup)
	return self.groups[unitgroup]:NewControl()
end

function uab.classes.ClickActivator.prototype:CopyToHostile(target, fromgroup)
    self.groups[fromgroup]:CopyTo(target.groups.hostile)
end

function uab.classes.ClickActivator.prototype:HasHostileActions()
    return self.groups.hostile:HasAnyActions()
end

function uab.classes.ClickActivator.prototype:GetHostileAction(idx)
    return self.groups.hostile:GetAction(idx)
end

function uab.classes.ClickActivator.prototype:CopyTo(target, fromgroup, togroup)
    --[[
    A deep copy, not including the name or activation click combination.
    Basically, copy the group data/actions for all groups
    --]]

    if fromgroup == "hostile" then
        if not togroup then
            togroup = fromgroup
        end
        self.groups.hostile:CopyTo(target.groups[togroup])
    else
        if not fromgroup then
            self.groups.hostile:CopyTo(target.groups.hostile)
        end

        uab.GroupIterate(
            function(self, context, name, info)
                if not context.fromgroup then
                    self.groups[info.name]:CopyTo(context.target.groups[info.name])
                elseif context.fromgroup == info.name then
                    if not context.togroup then
                        context.togroup = context.fromgroup
                    end
                    self.groups[info.name]:CopyTo(context.target.groups[context.togroup])
                    return true
                end
            end,
            self,
            { fromgroup = fromgroup, target = target, togroup = togroup} )
    end
end

function uab.classes.ClickActivator.prototype:IAmDead()
    uab.GroupIterate(
        function(self, context, name, info)
            self.groups[info.name]:IAmDead()
        end,
        self)
end

function uab.classes.ClickActivator.prototype:SetAction(group, number, data)
	if self.groups[group] then
	    self.groups[group]:SetAction(number, data)
	else
	    -- error : unknown group name
	end
end

function uab.classes.ClickActivator.prototype:ClearAction(group, number)
	if self.groups[group] then
	    self.groups[group]:ClearAction(number)
	else
	    -- error : unknown group name
	end
end

function uab.classes.ClickActivator.prototype:LayoutChanged(group)
    uab.GroupIterate(
        function(self, context, name, info)
            if not group or context == info.name then
                self.groups[info.name]:LayoutChanged()
                if context then
                    return true
                end
            end
        end,
        self,
        group )
end

function uab.classes.ClickActivator.prototype:Enable()
	self.data.enabled = true
    uab.GroupIterate(
        function(self, context, name, info)
            self.groups[info.name]:Enable()
        end,
        self)
end

function uab.classes.ClickActivator.prototype:Disable()
	self.data.enabled = nil
    uab.GroupIterate(
        function(self, context, name, info)
            self.groups[info.name]:Disable()
        end,
        self)
end

function uab.classes.ClickActivator.prototype:Enabled()
	return self.data.enabled
end

function uab.classes.ClickActivator.prototype:TimerEvent(elapsed)
    uab.GroupIterate(
        function(self, context, name, info)
            self.groups[info.name]:TimerEvent(elapsed)
        end,
        self)
end

function uab.classes.ClickActivator.prototype:RegisterFrame(frame)
	--[[
	Determine the correct group based on the unit associated with the frame.
	If no unit associated with the frame, then use the "partyraid" group.
	--]]

    local unit = SecureButton_GetModifiedUnit(frame)
    local group
    
    uab:Debug("Registering frame : '"..frame:GetName().."' in activator "..self.name)

    if unit then
        if unit == "player" or unit == "pet" then
            group = self.groups[unit]
        elseif "target" == string.sub(unit, 1, 6) then
            group = self.groups[unit]
        elseif "focus" == string.sub(unit, 1, 5) then
            group = self.groups[unit]
        else
            group = self.groups.partyraid
        end
    end
    
    if not group then
        group = self.groups.partyraid
    end

    group:RegisterFrame(frame)
end

function uab.classes.ClickActivator.prototype:UnregisterFrame(frame)
    uab.GroupIterate(
        function(self, context, name, info)
            if self.groups[info.name]:UnregisterFrame(context) then
                return true
            end
        end,
        self,
		frame)
end

function uab.classes.ClickActivator.prototype:HostileActionsChanged()
    uab.GroupIterate(
        function(self, context, name, info)
            self.groups[info.name]:HostileActionsChanged()
        end,
        self,
        nil)
end

function uab.classes.ClickActivator.prototype:GetSecureHeader()
    local header
    
    if uab.db.profile.activators[self.name].independent then
        header = uab.classes.SecureHeader:GetHeader(nil, self.name)
    else
        header = uab.classes.SecureHeader:GetHeader()
    end
    
    return header
end

function uab.classes.ClickActivator.prototype:SecureHeaderChanged(groupname)
    uab.GroupIterate(
        function(self, context, name, info)
            if not context or context == info.name then
                self.groups[info.name]:SecureHeaderChanged()
                if context then
                    return true
                end
            end
        end,
        self,
        groupname)
end

function uab.classes.ClickActivator.prototype:ScaleChanged(groupname)
    uab.GroupIterate(
        function(self, context, name, info)
            if not context or context == info.name then
                self.groups[info.name]:ScaleChanged()
                if context then
                    return true
                end
            end
        end,
        self,
        groupname)
end

function uab.classes.ClickActivator.prototype:PositionChanged(groupname)
    uab.GroupIterate(
        function(self, context, name, info)
            if not context or context == info.name then
                self.groups[info.name]:PositionChanged()
                if context then
                    return true
                end
            end
        end,
        self,
        groupname)
end

function uab.classes.ClickActivator.prototype:DirectCastChanged()
    uab.GroupIterate(
        function(self, context, name, info)
            self.groups[info.name]:DirectCastChanged()
        end,
        self)
end

function uab.classes.ClickActivator.prototype:StayOpenChanged()
    uab.GroupIterate(
        function(self, context, name, info)
            self.groups[info.name]:StayOpenChanged()
        end,
        self)
end

function uab.classes.ClickActivator.prototype:FastChangeUnitChanged()
    uab.GroupIterate(
        function(self, context, name, info)
            self.groups[info.name]:FastChangeUnitChanged()
        end,
        self)
end

function uab.classes.ClickActivator.prototype:HighlightBorderChanged()
    uab.GroupIterate(
        function(self, context, name, info)
            self.groups[info.name]:HighlightBorderChanged()
        end,
        self)
end

function uab.classes.ClickActivator.prototype:ClearActivationData()
end

--[[---------------------------------------------------------------------------
Helper methods, to be used only by the class itself
-----------------------------------------------------------------------------]]

function uab.classes.ClickActivator.prototype:SetActivationData(data)
    if data then
		self.data = data
	end
    
	if self.data.button == "LeftButton" then
		self.buttoncode = "1"
	elseif self.data.button == "RightButton" then
		self.buttoncode = "2"
	elseif self.data.button == "MiddleButton" then
		self.buttoncode = "3"
	elseif self.data.button == "Button4" then
		self.buttoncode = "4"
	else
		self.buttoncode = "5"
	end

    self.prefix = ""
    
	if self.data.prefixshift then
		self.prefix = "shift-" .. self.prefix
	end
	
	if self.data.prefixctrl then
		self.prefix = "ctrl-" .. self.prefix
	end
	
	if self.data.prefixalt then
		self.prefix = "alt-" .. self.prefix
	end
end
