uabFrameGroup = uab:NewModule("FrameGroup", "AceModuleCore-2.0")
uabFrameGroup:SetModuleMixins("AceDebug-2.0", "AceModuleCore-2.0")

uabFrameGroup.core = uab
uabFrameGroup.framegroups = {}

function uabFrameGroup:OnInitialize()
	self.super.OnInitialize(self)
end

function uabFrameGroup:OnEnable()
	self.super.OnEnable(self)
end

function uabFrameGroup:NewActivator(activator)
	for name, module in pairs(self.modules) do
		module:NewActivator(activator)
	end
end

function uabFrameGroup:GetFrameGroup(frame)
	for name, module in pairs(self.modules) do
        if module:IsFrameGroup(frame) then
			if module:IsEnabled() then
                return module
            end
		end
	end
end

function uabFrameGroup:HaveFrameControl(frame, activator)
	local framegroup = self:GetFrameGroup(frame)

	if framegroup then
	    return framegroup:HaveFrameControl(frame, activator)
	end
end

function uabFrameGroup:NextFrameControl(frame, activator)
	local framegroup = self:GetFrameGroup(frame)

	if framegroup then
        return framegroup:NextFrameControl(frame, activator)
	end
end

uabFrameGroup.FrameGroupScaleAndAnchor = uab.mixins.aceOO.Class()

function uabFrameGroup.FrameGroupScaleAndAnchor.prototype:init(profile)
    uabFrameGroup.FrameGroupScaleAndAnchor.super.prototype.init(self)
    self.profile = profile
end

function uabFrameGroup.FrameGroupScaleAndAnchor.prototype:IsDefaultScale()
    return self.profile.usedefaultscale
end

function uabFrameGroup.FrameGroupScaleAndAnchor.prototype:IsDefaultPosition()
    return self.profile.usedefaultanchor
end

function uabFrameGroup.FrameGroupScaleAndAnchor.prototype:UseDefaultScale()
    return {
        type = 'toggle',
        name = "Default scale",
        desc = "Use default scale value for this frame group",
        get = function()
            return self.profile.usedefaultscale
        end,
        set = function(v)
            if not v then
                if not self.profile.scale then
                    self.profile.scale = uab.db.profile.scale.default.value
                end
            end
        
            self.profile.usedefaultscale = v
            uab.main:ScaleChanged()
        end,
    }
end

function uabFrameGroup.FrameGroupScaleAndAnchor.prototype:UseDefaultAnchor()
    return {
        type = 'toggle',
        name = "Default anchor",
        desc = "Use default action button anchor information for this frame group",
        get = function()
            return self.profile.usedefaultanchor
        end,
        set = function(v)
            if not v then
                if not self.profile.point then
                    self.profile.point = uab.db.profile.position.default.point
                end
                
                if not self.profile.relpoint then
                    self.profile.relpoint = uab.db.profile.position.default.relpoint
                end
                
                if not self.profile.x then
                    self.profile.x = uab.db.profile.position.default.x
                end
                
                if not self.profile.y then
                    self.profile.y = uab.db.profile.position.default.y
                end
            end
            self.profile.usedefaultanchor = v
            uab.main:PositionChanged()
        end,
    }
end

function uabFrameGroup.FrameGroupScaleAndAnchor.prototype:GetScale()
    return self.profile.scale or uab.db.profile.scale.default.value
end

function uabFrameGroup.FrameGroupScaleAndAnchor.prototype:SetScale(v)
    self.profile.scale = v
end

function uabFrameGroup.FrameGroupScaleAndAnchor.prototype:GetPoint()
    return self.profile.point or uab.db.profile.position.default.point or "CENTER"
end

function uabFrameGroup.FrameGroupScaleAndAnchor.prototype:SetPoint(v)
    self.profile.point = v
end

function uabFrameGroup.FrameGroupScaleAndAnchor.prototype:GetRelPoint()
    return self.profile.relpoint or uab.db.profile.position.default.relpoint or "cursor"
end

function uabFrameGroup.FrameGroupScaleAndAnchor.prototype:SetRelPoint(v)
    self.profile.relpoint = v
end

function uabFrameGroup.FrameGroupScaleAndAnchor.prototype:GetHorizontal()
    return self.profile.x or uab.db.profile.position.default.x
end

function uabFrameGroup.FrameGroupScaleAndAnchor.prototype:SetHorizontal(v)
    self.profile.x = v
end

function uabFrameGroup.FrameGroupScaleAndAnchor.prototype:GetVertical()
    return self.profile.y or uab.db.profile.position.default.y
end

function uabFrameGroup.FrameGroupScaleAndAnchor.prototype:SetVertical(v)
    self.profile.y = v
end

--[[
FrameGroup module prototype

When implementing a FrameGroup module:
 - variables to set
 	- self.defaultDB to define your own default settings
 	    be sure to include the following settings as they are used to control
 	    the matching of frames, and the scaling and positioning of uab buttons
 	    in relation to these frames:
 	    - disabled
 	    - expression (used to match frames to this frame group)
 	    - usedefaultscale
		- usedefaultanchor
		- point, relpoint, x, y for default positioning, if not using the
		    uab defaults that the user defines.
 	- self.unitgroup to define the unitgroup with which this frame group is
	 	associated
 - OnInitialize functionality to implement
    - set self.defaultDB here if needed
    - call super.OnInitialize(self)
    - set any other variables you may need and perform any additional first-time
		initialization
 - Methods to implement
	- OnEnable (optional)
	- OnDisable (optional)
    - NewActivator
        Only override this if you need to support more than one frame per unit
		group for each activator, or if you need to support more than one unit
		group
	- HaveFrameControl(frame, activator)
        Only override this if you need to support more than one frame per unit
		group for each activator, or if you need to support more than one unit
		group
	- NextFrameControl(frame, activator)
        Only override this if you need to support more than one frame per unit
		group for each activator, or if you need to support more than one unit
		group
	- CreateFrameGroupOptions(name)
	    Only override this if you need to support more than one unit group, or
	    need to provide additional configuration options.  Be sure to call
	    CreateFrameGroupOption(name) to create the options for the frame groups
	    that you need.
	- OptionsForFrame(frame)
	    Override this if you need to support more than one unit group, since
	    the default implementation assumes a single expression to match the
	    frame with a frame group.
--]]

uabFrameGroup.modulePrototype.core = uabFrameGroup
uabFrameGroup.modulePrototype.controls = {}
uabFrameGroup.modulePrototype.frameGroupDefaultDB = {
    disabled = nil,
    expression = nil,
    usedefaultscale = true,
    usedefaultanchor = true,
}

function uabFrameGroup.modulePrototype:OnInitialize()
	if not self.db then
		self.core.core:RegisterDefaults(self.name, "profile", self.defaultDB or self.frameGroupDefaultDB)
		self.db = self.core.core:AcquireDBNamespace(self.name)
	end

    self.debugging = self.db.profile.debug

    self.core.core.options.args.lookandfeel.args.framegroups.args[self.name] = self:CreateFrameGroupOptions()
    
    self:SetupLayouts()
    
    self:SetDebugging(self.db.profile.debugging)
    self.core.core.options.args.debug.args[self.core.name..":"..self.name] = {
        type = 'toggle',
        name = self.core.name..":"..self.name,
        desc = "Enable debugging for the "..self.core.name..":"..self.name.." module",
        get = function()
            return self:IsDebugging()
        end,
        set = function(v)
            self.db.profile.debugging = v
            self:SetDebugging(self.db.profile.debugging)
        end,
    }
end

function uabFrameGroup.modulePrototype:IsEnabled()
    return not self.db.profile.disabled
end

function uabFrameGroup.modulePrototype:IsFrameGroup(frame)
    if not self.db.profile.disabled then
        local _, options = self:OptionsForFrame(frame)
        
        if options and not options.disabled then
            return true
        end
    end
end

function uabFrameGroup.modulePrototype:NewActivator(activator)
	self.controls[activator] = activator:NewUnitGroupControl(self.unitgroup)
end

function uabFrameGroup.modulePrototype:HaveFrameControl(frame, activator)
	local _, options = self:OptionsForFrame(frame)

	if options and not self.db.profile.disabled and not options.disabled then
	    return self.controls[activator]
	end
end

function uabFrameGroup.modulePrototype:NextFrameControl(frame, activator)
	local _, options = self:OptionsForFrame(frame)
	
	if options and not self.db.profile.disabled and not options.disabled then
	    local control = self.controls[activator]
	    
	    self.controls[activator] = nil
	    return control
	end
end

function uabFrameGroup.modulePrototype:CreateFrameGroupOptions()
    return self:CreateFrameGroupOption(self.db.profile)
end

function uabFrameGroup.modulePrototype:SetupLayouts()
    uabLayoutManager:LayoutChanged(self.db.profile.layout, self, self.name)
end

function uabFrameGroup.modulePrototype:OptionsForFrame(frame)
    if self.db.profile.disabled then return end
    
    local framename = frame:GetName()

    if self:MatchFrame(frame, self.db.profile) then
        return self.name, self.db.profile
    end
end

function uabFrameGroup.modulePrototype:MatchFrame(frame, defn)
    return frame and frame:GetName() and string.find(frame:GetName(), defn.expression)
end

function uabFrameGroup.modulePrototype:SetLayoutConfigureOptions(name, args)
	self.core.core.options.args.lookandfeel.args.framegroups.args[self.name].args.layout.args.configure = args
end

function uabFrameGroup.modulePrototype:CreateFrameGroupOption(profile, name, displayname)
    local options
    local me = self

    if not name then
        name = self.name
    end

    if not displayname then
        displayname = name
    end

    local scaleanchor = uabFrameGroup.FrameGroupScaleAndAnchor:new(profile)
    
    options = {
        type = 'group',
        name = name,
        desc = "Use, anchor, scale, and match expression for frames in this frame group",
        args = uab.ScaleAndAnchorOptions(scaleanchor)
    }

    options.args.enabled = {
        type = 'toggle',
        name = "Enabled",
        desc = "When checked, enable the use of UAB action sets for frames in the "..displayname.." frame group",
        order = 1,
        get = function()
            return not profile.disabled
        end,
        set = function(v)
            profile.disabled = not v
            uab.main:FrameGroupsChanged()
        end,
    }
            
    options.args.expression = {
        type = 'text',
        name = "Expression",
        usage = "<expression>",
        desc = "Define the match expression for frames in this frame group.  Any frame that has a name that matches this expression will use the anchor and scale settings defined for this frame group",
        order = 2,
        get = function()
            return profile.expression
        end,
        set = function(exp)
            profile.expression = exp
        end,
    }
    
    options.args.layout = {
        type = 'group',
        name = "Layout",
        desc = "Configure the layout for the action buttons when associated with frames in this frame group",
        order = 3,
        args = uabLayoutManager:FrameGroupLayoutOptions(self, name, profile)
    }
    
    return options
end

