uabLayoutManager = uab:NewModule("LayoutManager", "AceModuleCore-2.0")
uabLayoutManager:SetModuleMixins("AceDebug-2.0", "AceModuleCore-2.0")

uabLayoutManager.core = uab
uabLayoutManager.layouts = {}
uabLayoutManager.layoutswithdefault = {}

table.insert(uabLayoutManager.layoutswithdefault, "Use default")

function uabLayoutManager:OnInitialize()
	self.super.OnInitialize(self)

    self.core.options.args.lookandfeel.args.defaultanchorscale.args.layout = {
        type = 'group',
        name = "Layout",
        desc = "Select and configure button layout options",
		order = 10,
        args = {
            configure = {
                type = 'group',
                name = "Configure",
                desc = "Configure current layout options",
                order = 10,
                args = {}
            },
            select = {
                type = 'text',
                name = "Select",
                usage = "<name>",
                desc = "Select the current layout to use",
                order = 20,
                get = function()
                	local mgr = self:Get(self.core.db.profile.layout)
                	return mgr:GetDisplayName()
				end,
                set = function(name)
                	self.core.db.profile.layout = self:GetByDisplayName(name).name
                	self:LayoutChanged(self.core.db.profile.layout)
                end,
                validate = self.layouts
            },
        },
    }
end

function uabLayoutManager:AddLayout(displayname)
    table.insert(self.layouts, displayname)
    table.insert(self.layoutswithdefault, displayname)
end

function uabLayoutManager:OnEnable()
	self.super.OnEnable(self)

    self.core.GroupIterate(
        function(self, context, name, info)
            self.core.options.args.lookandfeel.args.groupanchorscale.args[info.name].args.layout = {
                type = 'group',
                name = "Layout",
                desc = "Configure the layout to use for action buttons in the group",
                order = 25,
                args = self:GroupLayoutOptions(info.name)
            }
        end,
        self,
        nil)
        
    self:LayoutChanged(self.core.db.profile.layout)
end

function uabLayoutManager:GetByDisplayName(displayname)
    for name, module in pairs(self.modules) do
        if module:GetDisplayName() == displayname then
            if module:IsEnabled() then
                return module
            end
        end
    end
end

function uabLayoutManager:Get(mgrname)
    for name, module in pairs(self.modules) do
        if name == mgrname then
            if module:IsEnabled() then
                return module
            end
        end
    end
end

function uabLayoutManager:LayoutSettingsChanged(activator)
    if not activator then
    	self.core.main:IterateActivators("LayoutChanged")
    else
        activator:LayoutChanged()
    end
end

function uabLayoutManager:LayoutContextName(group, name)
    if group and group.name then
        return "fgroup:"..group.name..":"..name
    end
    return "main"
end

function uabLayoutManager:LayoutChanged(proflayout, fgroup, name)
    if proflayout then
        local mgr = self:Get(proflayout)
        local layout
        if fgroup or not name then
            layout = mgr:GetLayout(self:LayoutContextName(fgroup, name))
        else
            layout = mgr:GetLayout("group:"..name)
        end
        
        local config = layout:GetConfig()
        local options = config:MenuOptionArgs()

        if fgroup then
            fgroup:SetLayoutConfigureOptions(name, options)
        elseif not name then
        	self.core.options.args.lookandfeel.args.defaultanchorscale.args.layout.args.configure.args = options
        else
        	self.core.options.args.lookandfeel.args.groupanchorscale.args[name].args.layout.args.configure.args = options
        end
    end

    self:LayoutSettingsChanged()
end

function uabLayoutManager:GroupLayoutOptions(groupname)
    local options = {
        configure = {
            type = 'group',
            name = "Configure",
            desc = "Configure current layout options for this group",
            disabled = function()
                if not self.core.db.profile.layouts[groupname] then
                    return true
                end
            end,
            order = 10,
            args = {}
        },
        select = {
            type = 'text',
            name = "Select",
            usage = "<name>",
            desc = "Select the layout to use for this group",
            order = 20,
            get = function()
                if not self.core.db.profile.layouts[groupname] then
                    return "Use default"
                else
                    local mgr = self:Get(self.core.db.profile.layouts[groupname])
                    return mgr:GetDisplayName()
                end
            end,
            set = function(name)
                if name == "Use default" then
                    self.core.db.profile.layouts[groupname] = nil
                else
                    self.core.db.profile.layouts[groupname] = self:GetByDisplayName(name).name
                end

                self:LayoutChanged(self.core.db.profile.layouts[groupname], nil, groupname)
            end,
            validate = self.layoutswithdefault
        },
    }

    return options
end

function uabLayoutManager:FrameGroupLayoutOptions(fgroup, fgname, profile)
    options = {
        configure = {
            type = 'group',
            name = "Configure",
            desc = "Configure current layout options",
            disabled = function()
                if not profile.layout then
                    return true
                end
            end,
            order = 10,
            args = {}
        },
        select = {
            type = 'text',
            name = "Select",
            usage = "<name>",
            desc = "Select the current layout to use",
            order = 20,
            get = function()
                if not profile.layout then
                    return "Use default"
                else
                    local mgr = self:Get(profile.layout)
                    return mgr:GetDisplayName()
                end
            end,
            set = function(name)
                if name == "Use default" then
                    profile.layout = nil
                else
                    profile.layout = self:GetByDisplayName(name).name
                end
                
                self:LayoutChanged(profile.layout, fgroup, fgname)
            end,
            validate = self.layoutswithdefault
        },
    }

    return options
end

--[[
LayoutManager module prototype

When implementing a LayoutManager module:
 - variables to set
 	- self.defaultDB to define your own default settings for all layouts, or
          to hold any settings you want to use.  Be sure to include the following
          settings as they are used to control the use of a LayoutManager:
 	    - disabled
 - OnInitialize functionality to implement
    - set self.defaultDB here if needed
    - call super.OnInitialize(self)
    - set any other variables you may need and perform any additional first-time
	initialization
 - Methods to implement
    - OnEnable (optional)
    - OnDisable (optional)
    - GetLayout(contextName) - required
        - Used to allocate a new layout in the specified context.  The settings
          to use are maintained by you related to the specified context.  If you
          have no settings for that context, you must initialize the returned layout
          to use default settings appropriate for that layout.
    - GetDisplayName() - required
        - Used to retrieve the name to appear in the FuBar drop-down menu for this
          layout manager.
--]]

uabLayoutManager.modulePrototype.core = uabLayoutManager
uabLayoutManager.modulePrototype.controls = {}
uabLayoutManager.modulePrototype.layoutManagerDefaultDB = {
    disabled = nil,
}

function uabLayoutManager.modulePrototype:OnInitialize()
    if not self.db then
        self.core.core:RegisterDefaults(self.name, "profile", self.defaultDB or self.layoutManagerDefaultDB)
        self.db = self.core.core:AcquireDBNamespace(self.name)
    end

    self.debugging = self.db.profile.debug
    if self:IsEnabled() then
	    self.core:AddLayout(self:GetDisplayName())
	end
    
    self:SetDebugging(self.db.profile.debugging)
    self.core.core.options.args.debug.args[self.core.name..":"..self.name] = {
        type = 'toggle',
        name = self.core.name..":"..self.name,
        desc = "Enable debugging for the "..self.core.name..":"..self.name.." module",
        get = function()
            return self:IsDebugging()
        end,
        set = function(v)
            self.db.profile.debugging = v
            self:SetDebugging(self.db.profile.debugging)
        end,
    }
end

function uabLayoutManager.modulePrototype:IsEnabled()
    return not self.db.profile.disabled
end
