uab.classes.SpellAction = uab.mixins.aceOO.Class(uab.classes.BaseAction)

function uab.classes.SpellAction.GetMaxRank(spellname)
	local idx = 0
    local maxrank
	local found
    local spellidx

    while not found do
		idx = idx + 1
        local name, rank = GetSpellName(idx, BOOKTYPE_SPELL)
        local texture

        if not name then
        	break
        end

        if name == spellname then
            if rank and string.find(rank, "Rank") then
                local _, _, thisrank = string.find(rank, "Rank (%d+)")
                
                if not maxrank or tonumber(thisrank) > maxrank then
                    maxrank = tonumber(thisrank)
                end
            else
                found = true
            end
		end
    end

    return "Rank "..maxrank
end

function uab.classes.SpellAction.prototype:init(data)
	uab.classes.SpellAction.super.prototype.init(self)
	
	self.data = data
	if self.data.max then
		self.text =	self.data.name
	elseif self.data.rank and string.find(self.data.rank, "Rank") then
		self.text = self.data.name..self.data.rank
	else
		self.text = self.data.name
	end

    self.attribs = {}

    self.attribs["type"] = "spell"
    self.attribs["spell"] = self.text
    
	self:SetIcon()
end

function uab.classes.SpellAction.prototype:GetAttributes()
    return self.attribs
end

function uab.classes.SpellAction.prototype:GetClassName()
	return "SpellAction"
end

function uab.classes.SpellAction.prototype:IsUsable()
    return IsUsableSpell(self:GetText())
end

function uab.classes.SpellAction.prototype:GetCooldown()
    local spellidx = self:GetSpellIdx()

    if spellidx then    
        return GetSpellCooldown(spellidx, BOOKTYPE_SPELL)
    end
end

function uab.classes.SpellAction.prototype:GetSpellName()
	return self.data.name
end

function uab.classes.SpellAction.prototype:IsRanked()
    local idx = self:GetSpellIdx()
    if idx then
        local name, rank = GetSpellName(idx, BOOKTYPE_SPELL)
        
        if rank and string.find(rank, "Rank") then
            return true
        end
    end
end

function uab.classes.SpellAction.prototype:ShowTooltip(frame)
    local spellidx = 0
    local found

    local start, last, rank = string.find(self.text, "Rank (%d+)")
    
    if start then
        -- not using max rank spell, so find this exact spell in the list

        while not found do
            spellidx = spellidx + 1
            local spellName, spellRank = GetSpellName(spellidx, BOOKTYPE_SPELL)

            if not spellName then
                break
            end

            if spellRank and string.find(spellRank, "Rank") then
                spellName = spellName .. "("..spellRank..")"
            end

            if spellName == self.text then
                found = true
            end
        end

        if not found then return end
    else
        -- using max rank spell, so find a matching spell, with the highest rank 

        local highestrank
        local highestrankidx
                    
        while not found do
            spellidx = spellidx + 1
            local spellName, spellRank = GetSpellName(spellidx, BOOKTYPE_SPELL)

            if not spellName then
                break
            end

            if spellName == self.text then
                if spellRank and string.find(spellRank, "Rank") then
                    _, _, rank = string.find(spellRank, "Rank (%d+)")

                    rank = tonumber(rank)
                    if not highestrank or rank > highestrank then
                        highestrank = rank
                        highestrankidx = spellidx
                    end                        
                else
                    found = true
                    highestrankidx = spellidx
                end
            end
        end

        if not highestrankidx then return end
        spellidx = highestrankidx
    end

    GameTooltip:SetOwner(frame, "ANCHOR_RIGHT")
    GameTooltip:SetBackdropColor(0.1, 0.1, 0.1, 0.8)
    
    local name, srank = GetSpellName(spellidx, BOOKTYPE_SPELL)
    
    GameTooltip:SetSpell(spellidx, SpellBookFrame.bookType)
    if srank and string.find(srank, "Rank") then
        if not GameTooltipTextRight1:GetText() or not GameTooltipTextRight1:IsVisible() then
            GameTooltipTextRight1:SetText(srank)
            GameTooltipTextRight1:Show()
            GameTooltipTextRight1:ClearAllPoints()
            GameTooltipTextRight1:SetPoint("TOPRIGHT", GameTooltip, "TOPRIGHT", -10, -10)
            GameTooltipTextRight1:SetTextColor(0.5, 0.5, 0.5, 1)
        end
    else
        GameTooltipTextRight1:SetText()
        GameTooltipTextRight1:Hide()
    end
end

--[[---------------------------------------------------------------------------
Helper methods, to be used only by the class itself
-----------------------------------------------------------------------------]]

function uab.classes.SpellAction.prototype:SetIcon()
    self.icon = self.data.icon

    if not self.icon then    
        local spellidx = self:GetSpellIdx()
    
        if spellidx then
            self.icon = GetSpellTexture(spellidx, BOOKTYPE_SPELL)
        end
    end
    
    self.data.icon = self.icon
end

function uab.classes.SpellAction.prototype:GetSpellIdx()
	--[[
	For some spells, the icon is the same, regardless of the rank of spell.  For
    others, the icon changes based on the rank.  To get the icon, and cover all
    cases, find the spell of the correct rank, or the highest rank, if max.
	--]]

	local idx = 0
    local maxrank
	local found
    local spellidx

    while not found do
		idx = idx + 1
        local name, rank = GetSpellName(idx, BOOKTYPE_SPELL)
        local texture

        if not name then
        	break
        end

        if name == self.data.name then
            if rank and string.find(rank, "Rank") then
                if self.rank and rank == self.rank then
                    spellidx = idx
                    found = true
                elseif not self.rank then
                    local _, _, thisrank = string.find(rank, "Rank (%d+)")
                    
                    if not maxrank or tonumber(thisrank) > maxrank then
                        maxrank = tonumber(thisrank)
                        spellidx = idx
                    end
                end
            else
                spellidx = idx
                found = true
            end
		end
    end

    if spellidx then
        return spellidx
    end
end

