--[[
Class to manage the actions tab of the config dialog for the add-on.
--]]

uab.classes.CfgActionButton = uab.mixins.aceOO.Class()

function uab.classes.CfgActionButton.prototype:init(actionhdr, info, num, relinfo)
    uab.classes.CfgActionButton.super.prototype.init(self)

    self.info = info
    if not info then
        self.info = {}
    end
    
    self.actionhdr = actionhdr
    self.effects = "Default"
    self.num = num
    self.frame = CreateFrame("CheckButton", actionhdr:GetActionsFrame():GetName().."Action"..num, actionhdr:GetActionsFrame(), "UAB_ActionButtonTemplate")
    self.frame:ClearAllPoints()
    self.frame:SetPoint(relinfo.point, relinfo.relto, relinfo.relpoint, relinfo.x, relinfo.y)
    self.frame.uabobj = self

    getglobal(self.frame:GetName().."ActionNum"):SetText(num..".")
    fontName, fontHeight, fontFlags = getglobal(self.frame:GetName().."ActionNum"):GetFont()
    
    getglobal(self.frame:GetName().."ActionNum"):SetFont(fontName, 10, fontFlags)
    
    getglobal(self.frame:GetName().."Icon"):SetTexture("Interface\\AddOns\\UnitActionBars\\images\\EmptySlot")
    self.frame:RegisterForDrag("LeftButton")
    self.frame:SetBackdropColor(0, 0, 0)
    
    self.frame:SetScript("OnEnter", function(self, motion) self.uabobj:ShowTooltip() end)
    self.frame:SetScript("OnReceiveDrag", function(self) self.uabobj:OnReceiveDrag() end)
    self.frame:SetScript("OnClick", function(self, button, down) self.uabobj:OnClick(button) end)
    self.frame:SetScript("OnLeave", function(self, motion) GameTooltip:Hide() end)
end    

function uab.classes.CfgActionButton.prototype:GetNumber()
    return self.num
end

function uab.classes.CfgActionButton.prototype:GetActionData()
    return self.info
end

function uab.classes.CfgActionButton.prototype:SetInfo(info)
    self.info = info

    if not self.info then
        self.info = {}
    end
    
    self:ResetAction()
    self.actionhdr:SetActionData(self)
end

function uab.classes.CfgActionButton.prototype:SetEffect(effect)
    self.effects = effect
    self:RefreshActionControls()
end

function uab.classes.CfgActionButton.prototype:Clear()
    self.info.classname = nil
    self.info.data = {}
    
    self:ResetAction()
    self.actionhdr:SetActionData(self)
end

function uab.classes.CfgActionButton.prototype:GetFrame()
    return self.frame
end
                
function uab.classes.CfgActionButton.prototype:ShowTooltip()
    if self.action then
        self.action:ShowTooltip(self.frame)
    end
end

function uab.classes.CfgActionButton.prototype:OnClick()
    local checked = nil

    checked = self.frame:GetChecked()
    
    self.actionhdr:ClearSelected()

    if arg1 == "LeftButton" and self:ReceiveDrop() then
        checked = true
        
        self.info.effect = {}
        
        self:ResetAction()
    end

    if checked then
        self.actionhdr:SetActionData(self)
        self.actionhdr:SelectAction(self)
    end
end

function uab.classes.CfgActionButton.prototype:OnReceiveDrag()
    local checked = self:ReceiveDrop()

    if checked then
        self.info.effect = {}
    
        self:ResetAction()
        self.actionhdr:SetActionData(self)
        self.actionhdr:SelectAction(self)
    end
end

function uab.classes.CfgActionButton.prototype:ReceiveDrop()
    local checked
    
    if CursorHasSpell() then
        local _, slot, spelltype = GetCursorInfo()

        if spelltype == "spell" then
            self:AssignSpell(slot)
            checked = 1
        end
    elseif CursorHasItem() then
        local _, id, link = GetCursorInfo()
        
        self:AssignItem(id, link)
        checked = 1
    elseif CursorHasMacro() then
        local _, index, _ = GetCursorInfo()
        
        self:AssignMacro(index)
        checked = 1
    end
    
    ClearCursor()
    
    return checked
end

function uab.classes.CfgActionButton.prototype:AssignSpell(slot)
    local name, rank = GetSpellName(slot, BOOKTYPE_SPELL)

    self.info.classname = "SpellAction"
    self.info.data = {}
    self.info.data.max = true
    self.info.data.name = name
end

function uab.classes.CfgActionButton.prototype:AssignItem(id, link)
    local sName, sLink, iRarity, iLevel, iMinLevel, sType, sSubType, iStackCount = GetItemInfo(id)

    self.info.classname = "ItemAction"
    self.info.data = {}
    self.info.data.name = sName
end

function uab.classes.CfgActionButton.prototype:AssignMacro(index)
    local name, texture, macrobody, localVar = GetMacroInfo(index)

    self.info.classname = "MacroAction"
    self.info.data = {}
    self.info.data.name = name
end

function uab.classes.CfgActionButton.prototype:ResetAction()
    local icon = getglobal(self.frame:GetName().."Icon")

    if self.info.classname then
        self.action = uab.classes[self.info.classname]:new(self.info.data)
            
        icon:SetTexture(self.action:GetIcon())
    else
        self.action = nil
        icon:SetTexture("Interface\\AddOns\\UnitActionBars\\images\\EmptySlot")
    end
end

function uab.classes.CfgActionButton.prototype:SetCurrent(current)
    self.frame:SetChecked(current)
    
    if current then
        self:RefreshActionControls()
    else
        self.frame:SetButtonState("NORMAL")
    end
end

function uab.classes.CfgActionButton.prototype:GetRanks()
    local _, _, maxrank = string.find(uab.classes.SpellAction.GetMaxRank(self.info.data.name), "Rank (%d+)")
    local ranks = {}
    local data = {}
    
    data.text = "Max"
    data.func = uab.DropDownSelect
    data.arg1 = self
    data.arg2 = { func = "SelectRank", arg1 = "Max" }

    table.insert(ranks, data)
    
    for idx = 1, maxrank, 1 do
        data = {}
        
        data.text = "Rank "..idx
        data.func = uab.DropDownSelect
        data.arg1 = self
        data.arg2 = { func = "SelectRank", arg1 = "Rank "..idx }

        table.insert(ranks, data)
    end
    
    return ranks
end

function uab.classes.CfgActionButton.prototype:FillUseRank()
    if self.info and self.info.classname == "SpellAction" and self.action:IsRanked() then
        local fname = self.actionhdr:GetActionsFrame()
        local userank = getglobal(fname:GetName().."UseRank")
        
        local _, _, maxrank = string.find(uab.classes.SpellAction.GetMaxRank(self.info.data.name), "Rank (%d+)")

        if maxrank then
            local ranks = self:GetRanks()
            
            UIDropDownMenu_AddButton(ranks[1])
            if self.info.data.max then
                selected = 1
            end        
    
        	for idx = 2, # ranks, 1 do
                UIDropDownMenu_AddButton(ranks[idx])
                if not self.info.data.max and self.info.data.rank == "("..ranks[idx].arg2.arg1..")" then
                    selected = idx
                end        
        	end
        
            UIDropDownMenu_SetSelectedID(userank, selected)
        end        
    end
end

function uab.classes.CfgActionButton.prototype:SelectRank(selector, selection)
    local fname = self.actionhdr:GetActionsFrame()
    local userank = getglobal(fname:GetName().."UseRank")
    
    UIDropDownMenu_SetSelectedID(userank, selector:GetID())
    
    if selection == "Max" then
        self.info.data.max = true
        self.info.data.rank = nil
    else
        self.info.data.max = nil
        self.info.data.rank = "("..selection..")"
    end
    
    self:ResetAction()
end

function uab.classes.CfgActionButton.prototype:UnitEffectOnClick(frame, uniteffect)
	if not self.info.effect[self.effects] and frame:GetChecked() then
		self.info.effect[self.effects] = {}
	end
	
    if frame:GetChecked() then
		self.info.effect[self.effects][uniteffect] = true
	elseif self.info.effect[self.effects] then
		self.info.effect[self.effects][uniteffect] = nil
	end
	
    self.actionhdr:SetActionData(self)
end

function uab.classes.CfgActionButton.prototype:RefreshActionControls()
    local fname = self.actionhdr:GetActionsFrame()

    local clear = getglobal(fname:GetName().."ActionClear")
    local userank = getglobal(fname:GetName().."UseRank")
    local uniteffectlbl = getglobal(fname:GetName().."UnitEffectLabel")
    local uniteffect = getglobal(fname:GetName().."UnitEffect")
    local effects = getglobal(fname:GetName().."ActionEffects")

    if self.action then
        uniteffectlbl:Show()
        clear:Enable()
        clear:Show()
        
        if self.info.classname == "SpellAction" then
            if self.action:IsRanked() then
                userank:Enable()
                uab.classes.ActionsTab.InitDropdown = self.actionhdr
                UIDropDownMenu_Initialize(userank,
                    function()
                        if uab.classes.ActionsTab.InitDropdown then
                            uab.classes.ActionsTab.InitDropdown:FillUseRank()
                        else
                            this:GetParent().uabobj:FillUseRank()
                        end
                    end)
                uab.classes.ActionsTab.InitDropdown = nil

                local ranks = self:GetRanks()
                local selected
                
                if self.info.data.max then
                    selected = 1
                else
                	for idx = 2, # ranks, 1 do
                        if self.info.data.rank == "("..ranks[idx].arg2.arg1..")" then
                            selected = idx
                        end        
                	end
                end        
            
                if selected then
                    UIDropDownMenu_SetSelectedName(userank, ranks[selected].arg2.arg1)
                end
                
                userank:Show()
            else
                userank:Hide()
            end
        else
            userank:Hide()
        end
        
        uniteffect:Show()
        effects:Show()

        getglobal(effects:GetName().."BorderHighlight"):SetChecked(self.info.effect[self.effects] and self.info.effect[self.effects].highlight)
        getglobal(effects:GetName().."Alpha50"):SetChecked(self.info.effect[self.effects] and self.info.effect[self.effects].lowalpha)
        getglobal(effects:GetName().."Blink"):SetChecked(self.info.effect[self.effects] and self.info.effect[self.effects].blink)
        getglobal(effects:GetName().."Hide"):SetChecked(self.info.effect[self.effects] and self.info.effect[self.effects].noalpha)
        getglobal(effects:GetName().."NotUsable"):SetChecked(self.info.effect[self.effects] and self.info.effect[self.effects].notusable)
    else
        clear:Hide()
        userank:Hide()
        uniteffectlbl:Hide()
        uniteffect:Hide()
        effects:Hide()
    end
end

