local uabagufFrameGroup = uabFrameGroup:NewModule("ag_UnitFrames")

uabagufFrameGroup.controls = {}
uabagufFrameGroup.defaultDB = {
    framegroups = {
        ["Player"] = {
            expression = "^aUFplayer$",
            usedefaultscale = true,
            usedefaultanchor = true,
            unitgroup = "player",
        },
        ["Player Pet"] = {
            expression = "^aUFpet$",
            usedefaultscale = true,
            usedefaultanchor = true,
            unitgroup = "pet",
        },
        ["Party"] = {
            expression = "^aUFgroupparty.+",
            usedefaultscale = true,
            usedefaultanchor = true,
            unitgroup = "partyraid",
        },
        ["Raid"] = {
            expression = "^aUFgroupgroup.+",
            usedefaultscale = true,
            usedefaultanchor = true,
            unitgroup = "partyraid",
        },
        ["Focus"] = {
            expression = "^aUFfocus$",
            usedefaultscale = true,
            usedefaultanchor = true,
            unitgroup = "focus",
        },
        ["Focus Target"] = {
            expression = "^aUFfocustarget$",
            usedefaultscale = true,
            usedefaultanchor = true,
            unitgroup = "focustarget",
        },
        ["Target"] = {
            expression = "^aUFtarget$",
            usedefaultscale = true,
            usedefaultanchor = true,
            unitgroup = "target",
        },
        ["Target of Target"] = {
            expression = "^aUFtargettarget$",
            usedefaultscale = true,
            usedefaultanchor = true,
            unitgroup = "targettarget",
        },
        ["ToT Target"] = {
            expression = "^aUFtargettargettarget$",
            usedefaultscale = true,
            usedefaultanchor = true,
            unitgroup = "targettargettarget",
        },
    },
}

function uabagufFrameGroup:OnInitialize()
    self.super.OnInitialize(self)
end

function uabagufFrameGroup:OnEnable()
end

function uabagufFrameGroup:OnDisable()
end

function uabagufFrameGroup:NewActivator(activator)
	self.controls[activator] = {}
    
    self:AllocateControlsForUnitGroup(activator, "player")
    self:AllocateControlsForUnitGroup(activator, "pet")
    self:AllocateControlsForUnitGroup(activator, "partyraid")
    self:AllocateControlsForUnitGroup(activator, "partyraid")
    self:AllocateControlsForUnitGroup(activator, "focus")
    self:AllocateControlsForUnitGroup(activator, "focustarget")
    self:AllocateControlsForUnitGroup(activator, "target")
    self:AllocateControlsForUnitGroup(activator, "targettarget")
    self:AllocateControlsForUnitGroup(activator, "targettargettarget")
end

function uabagufFrameGroup:AllocateControlsForUnitGroup(activator, groupname)
    if not self.controls[activator][groupname] then
        self.controls[activator][groupname] = {}
    end
    
	for idx = 1, uab.constants.groupInfo[groupname].unitcount do
        table.insert(self.controls[activator][groupname], activator:NewUnitGroupControl(groupname))
    end
end

function uabagufFrameGroup:HaveFrameControl(frame, activator)
    if self.db.profile.disabled then return end

	local _, options = self:OptionsForFrame(frame)
	
	if not options or options.disabled then return end

	if self.controls[activator] and self.controls[activator][options.unitgroup] then
		return # self.controls[activator][options.unitgroup] > 0
	end
end

function uabagufFrameGroup:NextFrameControl(frame, activator)
	if self.db.profile.disabled then return end

	local _, options = self:OptionsForFrame(frame)

	if not options or options.disabled then return end

	local control
	
    if self.controls[activator] and self.controls[activator][options.unitgroup]
        and # self.controls[activator][options.unitgroup] > 0 then
		control = table.remove(self.controls[activator][options.unitgroup])
	end

	return control
end

function uabagufFrameGroup:OptionsForFrame(frame)
    if self.db.profile.disabled then return end
    
    for name, defn in pairs(self.db.profile.framegroups) do
        if self:MatchFrame(frame, defn) then
            return name, defn
        end
    end
end

function uabagufFrameGroup:CreateFrameGroupOptions()
    local args = {}

    args = {
        type = 'group',
        name = "ag_UnitFrames",
        desc = "Use, anchor, scale, and match expression for ag_UnitFrames frames",
        args = {
            enabled = {
                type = 'toggle',
                name = "Enabled",
                desc = "When checked, enable the use of UAB action sets for ag_UnitFrames frames",
                order = 1,
                get = function()
                    return not self.db.profile.disabled
                end,
                set = function(v)
                    self.db.profile.disabled = not v
                    self.core.core.main:FrameGroupsChanged()
                end,
            },
            
            spacing3 = {
                name = " ",
                type = 'header',
                order = 5,
            },
        
        },
    }
    
    args.args["Player"] = self:CreateFrameGroupOption(self.db.profile.framegroups["Player"], "Player", "ag_UnitFrames Player")
    args.args["Player"].disabled = function() return self.db.profile.disabled end
    args.args["Player"].order = 10
    
    args.args["Player Pet"] = self:CreateFrameGroupOption(self.db.profile.framegroups["Player Pet"], "Player Pet", "ag_UnitFrames Player Pet")
    args.args["Player Pet"].disabled = function() return self.db.profile.disabled end
    args.args["Player Pet"].order = 20
    
    args.args["Party"] = self:CreateFrameGroupOption(self.db.profile.framegroups["Party"], "Party", "ag_UnitFrames Party")
    args.args["Party"].disabled = function() return self.db.profile.disabled end
    args.args["Party"].order = 30
    
    args.args["Raid"] = self:CreateFrameGroupOption(self.db.profile.framegroups["Raid"], "Raid", "ag_UnitFrames Raid")
    args.args["Raid"].disabled = function() return self.db.profile.disabled end
    args.args["Raid"].order = 30
    
    args.args["Focus"] = self:CreateFrameGroupOption(self.db.profile.framegroups["Focus"], "Focus", "ag_UnitFrames Focus")
    args.args["Focus"].disabled = function() return self.db.profile.disabled end
    args.args["Focus"].order = 40
    
    args.args["Focus Target"] = self:CreateFrameGroupOption(self.db.profile.framegroups["Focus Target"], "Focus Target", "ag_UnitFrames Focus Target")
    args.args["Focus Target"].disabled = function() return self.db.profile.disabled end
    args.args["Focus Target"].order = 50
    
    args.args["Target"] = self:CreateFrameGroupOption(self.db.profile.framegroups["Target"], "Target", "ag_UnitFrames Target")
    args.args["Target"].disabled = function() return self.db.profile.disabled end
    args.args["Target"].order = 60
    
    args.args["Target of Target"] = self:CreateFrameGroupOption(self.db.profile.framegroups["Target of Target"], "Target of Target", "ag_UnitFrames Target of Target")
    args.args["Target of Target"].disabled = function() return self.db.profile.disabled end
    args.args["Target of Target"].order = 70
    
    args.args["ToT Target"] = self:CreateFrameGroupOption(self.db.profile.framegroups["ToT Target"], "ToT Target", "ag_UnitFrames ToT Target")
    args.args["ToT Target"].disabled = function() return self.db.profile.disabled end
    args.args["ToT Target"].order = 80
    
    return args
end

function uabagufFrameGroup:SetupLayouts()
    uabLayoutManager:LayoutChanged(self.db.profile.framegroups["Player"].layout, self, "Player")
    uabLayoutManager:LayoutChanged(self.db.profile.framegroups["Player Pet"].layout, self, "Player Pet")
    uabLayoutManager:LayoutChanged(self.db.profile.framegroups["Party"].layout, self, "Party")
    uabLayoutManager:LayoutChanged(self.db.profile.framegroups["Raid"].layout, self, "Raid")
    uabLayoutManager:LayoutChanged(self.db.profile.framegroups["Focus"].layout, self, "Focus")
    uabLayoutManager:LayoutChanged(self.db.profile.framegroups["Focus Target"].layout, self, "Focus Target")
    uabLayoutManager:LayoutChanged(self.db.profile.framegroups["Target"].layout, self, "Target")
    uabLayoutManager:LayoutChanged(self.db.profile.framegroups["Target of Target"].layout, self, "Target of Target")
    uabLayoutManager:LayoutChanged(self.db.profile.framegroups["ToT Target"].layout, self, "ToT Target")
end

function uabagufFrameGroup:SetLayoutConfigureOptions(name, args)
    self.core.core.options.args.lookandfeel.args.framegroups.args[self.name].args[name].args.layout.args.configure.args = args
end

