local uabStatus_MainDebuffs = uabUnitStatus:NewModule("Main Debuffs")

uabStatus_MainDebuffs.aura = AceLibrary("SpecialEvents-Aura-2.0")
uabStatus_MainDebuffs.defaultDB = {
    disabled = nil,
    curse = {
        disabled = nil,
        priority = 50,
        color = { r = 0.75, g = 0, b = 1, a = 1, },
        classfilter = {
            druid = true,
            hunter = true,
            mage = true,
            paladin = true,
            priest = true,
            rogue = true,
            shaman = true,
            warlock = true,
            warrior = true,
        },
	},
    disease = {
        disabled = nil,
        priority = 50,
        color = { r = 1,    g = 0.75, b = 0, a = 1, },
        classfilter = {
            druid = true,
            hunter = true,
            mage = true,
            paladin = true,
            priest = true,
            rogue = true,
            shaman = true,
            warlock = true,
            warrior = true,
        },
	},
    magic = {
        disabled = nil,
        priority = 50,
        color = { r = 0,    g = 0.25, b = 1, a = 1, },
        classfilter = {
            druid = true,
            hunter = true,
            mage = true,
            paladin = true,
            priest = true,
            rogue = true,
            shaman = true,
            warlock = true,
            warrior = true,
        },
	},
    poison = {
        disabled = nil,
        priority = 50,
        color = { r = 0,    g = 1,    b = 0, a = 1, },
        classfilter = {
            druid = true,
            hunter = true,
            mage = true,
            paladin = true,
            priest = true,
            rogue = true,
            shaman = true,
            warlock = true,
            warrior = true,
        },
	},
}

uabStatus_MainDebuffs.constants = {
    strings = {
        dcurse = "Debuff: Curse",
        ddisease = "Debuff: Disease",
        dmagic = "Debuff: Magic",
        dpoison = "Debuff: Poison",
        
        curse = "Curse",
        disease = "Disease",
        magic = "Magic",
        poison = "Poison",
        
    }
}

function uabStatus_MainDebuffs:OnInitialize()
    self.super.OnInitialize(self)

	self:RegisterUnitStatus(uabStatus_MainDebuffs.constants.strings.dcurse)
	self:RegisterUnitStatus(uabStatus_MainDebuffs.constants.strings.ddisease)
	self:RegisterUnitStatus(uabStatus_MainDebuffs.constants.strings.dmagic)
	self:RegisterUnitStatus(uabStatus_MainDebuffs.constants.strings.dpoison)
end

function uabStatus_MainDebuffs:OnEnable()
end

function uabStatus_MainDebuffs:OnDisable()
end

function uabStatus_MainDebuffs:IsStatusGained(name, unit, action)
    local class = self:GetUnitClass(unit)
    local has
	
    if name == uabStatus_MainDebuffs.constants.strings.dcurse then
        if not class or self.db.profile.curse.classfilter[class] then
            has = self.aura:UnitHasDebuffType(unit, uabStatus_MainDebuffs.constants.strings.curse)
        end
    elseif name == uabStatus_MainDebuffs.constants.strings.ddisease then
        if not class or self.db.profile.disease.classfilter[class] then
       	    has = self.aura:UnitHasDebuffType(unit, uabStatus_MainDebuffs.constants.strings.disease)
    	end
    elseif name == uabStatus_MainDebuffs.constants.strings.dmagic then
        if not class or self.db.profile.magic.classfilter[class] then
            has = self.aura:UnitHasDebuffType(unit, uabStatus_MainDebuffs.constants.strings.magic)
    	end
    elseif name == uabStatus_MainDebuffs.constants.strings.dpoison then
        if not class or self.db.profile.poison.classfilter[class] then
    	    has = self.aura:UnitHasDebuffType(unit, uabStatus_MainDebuffs.constants.strings.poison)
    	end
    end

    return has
end

function uabStatus_MainDebuffs:OptionsForUnitStatus(name)
	if name == uabStatus_MainDebuffs.constants.strings.dcurse then
		return self.db.profile.curse
	elseif name == uabStatus_MainDebuffs.constants.strings.ddisease then
		return self.db.profile.disease
	elseif name == uabStatus_MainDebuffs.constants.strings.dmagic then
		return self.db.profile.magic
	elseif name == uabStatus_MainDebuffs.constants.strings.dpoison then
		return self.db.profile.poison
	end
end

function uabStatus_MainDebuffs:CreateUnitStatusOptions()
    local args = {}

    args = {
        type = 'group',
        name = "Debuff",
        desc = "Configure the options for debuff unit status values",
        args = {
            enabled = {
                type = 'toggle',
                name = "Enabled",
                desc = "When checked, enable the use of the debuff unit status values",
                order = 1,
                get = function()
                    return not self.db.profile.disabled
                end,
                set = function(v)
                    self.db.profile.disabled = not v
                    self.core.core.main:FrameGroupsChanged()
                end,
            },

            spacing3 = {
                name = " ",
                type = 'header',
                order = 5,
            },

        },
    }

    args.args["curse"] = self:CreateUnitStatusOption(self.db.profile.curse, uabStatus_MainDebuffs.constants.strings.dcurse, uabStatus_MainDebuffs.constants.strings.dcurse, true)
    args.args["curse"].order = 10

    args.args["disease"] = self:CreateUnitStatusOption(self.db.profile.disease, uabStatus_MainDebuffs.constants.strings.ddisease, uabStatus_MainDebuffs.constants.strings.ddisease, true)
    args.args["disease"].order = 20

    args.args["magic"] = self:CreateUnitStatusOption(self.db.profile.magic, uabStatus_MainDebuffs.constants.strings.dmagic, uabStatus_MainDebuffs.constants.strings.dmagic, true)
    args.args["magic"].order = 30

    args.args["poison"] = self:CreateUnitStatusOption(self.db.profile.poison, uabStatus_MainDebuffs.constants.strings.dpoison, uabStatus_MainDebuffs.constants.strings.dpoison, true)
    args.args["poison"].order = 40

    return args
end
